
#include "Vector.h"
#include "Object.h"

#define stdinc 25

Vector::Vector()
{
  incrementSize = stdinc;
  elementCount = incrementSize-1;
  elementData = 0;
  vec = new (Object *)[incrementSize];
}

Vector::Vector(int _num)
{
  incrementSize = stdinc;
  elementCount = _num-1;
  elementData = 0;
  vec = new (Object *)[_num];
}

Vector::Vector(int _num, int _inc)
{
  incrementSize = _inc;
  elementCount  = _num-1;
  elementData = 0;
  vec = new (Object *)[_num];
}

Vector::Vector(Vector &_cvec)
{
  incrementSize = _cvec.getIncrementSize();
  elementCount = _cvec.getElementCount();
  elementData = _cvec.size();
  vec= new (Object *)[elementCount];
  for (int i=1; i<=elementData; i++)
    {
      vec[i]=_cvec.elementAt(i);
    }
}

Vector::~Vector()
{
  delete vec;
}

void Vector::resize()
{
  Object **newvec;
  elementCount += incrementSize;
  newvec = new (Object *)[elementCount];
  for (int i=0; i<=elementData; i++)
    {
      newvec[i] = vec[i];
    }
  delete vec;
  vec = newvec;
}

void Vector::trim()
{
  if (elementData)
    {
      Object **newvec;
      elementCount -= incrementSize;
      newvec = new (Object *)[elementCount];
      for (int i=0; i<=elementData; i++)
	{
	  newvec[i] = vec[i];
	}
      delete vec;
      vec = newvec;
    }
}
      
void Vector::addElement(Object *_last)
{
  if (elementData == (elementCount-1)) resize();
  ++elementData;
  vec[elementData] = _last;
}

void Vector::insertElementAt(Object *_mid, int _where)

{
  if (_where <= elementData)
    {
      if (elementData == (elementCount-1)) resize();
      for (int i=elementData; i>=_where; i--)
	{
	  vec[i+1] = vec[i];
	}
      ++elementData;
      vec[_where] = _mid;
    }
  else 
    {
      addElement(_mid);
    }
}

void Vector::setElementAt(Object *_mid, int _where)
{
  if (_where<=elementData)
    {
      delete vec[_where];
      vec[_where]=_mid;
    }
  else ; //Exeption werfen
}

bool Vector::removeElement(Object *_what)
{
  int i=elementData;
  while(i && !(vec[i]->equals(_what))) --i;
  if (i) 
    {
      removeElementAt(i);
      return true;
    }
  else return false;
}

bool Vector::removeElementAt(int _where)
{
  if (_where<=elementData)
    {
      --elementData;
      for (int i = _where; i<=elementData; i++)
	{
	  vec[i] = vec[i+1];
	}
      if ((elementCount-elementData) > incrementSize) trim();
      return true;
    }
  else return false;
}

Object *Vector::elementAt(int _which)
{
  if (_which <= elementData)
    {
      return vec[_which];
    }
  else 
    {
      return NULL; //error (z.Z.: Speicher wird nicht freigegeben)
    }
}

bool Vector::isEmpty()
{
  if (elementData > 0) return false;
  else return true;
}

String Vector::toString()
{
  String vectext;
  for (int i=1; i < elementData; i++)
    {
      vectext += vec[i]->toString();
    }
  vectext += vec[elementData]->toString();
  return vectext;
}

int Vector::size()
{
  return elementData;
}

void Vector::setIncrementSize(int _size)
{
  if (_size) incrementSize=_size;
}

int Vector::getIncrementSize()
{
  return incrementSize;
}

int Vector::getElementCount()
{
  return elementCount;
}

void Vector::ensureCapacity(int _cap)
{
  if (_cap>elementCount)
    {
      Object **newvec;
      elementCount = _cap;
      newvec = new (Object *)[elementCount];
      for (int i=0; i<=elementData; i++)
	{
	  newvec[i] = vec[i];
	}
      delete vec;
      vec = newvec;
    }
}

Enumeration *Vector::elements()
{
  Enumeration *elms=(Enumeration *)this;
  readElements = 0;
  return elms;
}

Object *Vector::nextElement()
{
  if (elementData-readElements) 
    {
      ++readElements;
      return vec[readElements];
    }
  else return NULL;
}

bool Vector::hasMoreElements()
{
  if(elementData-readElements == 0) return false;
  else return true;
}

bool Vector::equals(Vector *_peer)
{
  int i=elementData;
  bool comp=true;
  while(i && comp)
    {
      if (!((_peer->elementAt(i))->equals(vec[i]))) comp=false;
      --i;
    }
  return comp;
}


ostream & operator<<(ostream &os,Vector &obj)
{
  os << obj.toString(); 
  return os;
}






